--[[
Name: PostalLib-1.0
Revision: $Rev: 1 $
Author(s): Grennon (jeramy.smith@gmail.com)
Description: A library that provides basic mail management functions.
Dependencies: AceLibrary
]]--

local MAJOR_VERSION = "PostalLib-1.0"
local MINOR_VERSION = "$Revision: 1 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0") end
if not AceLibrary:HasInstance("AceHook-2.1") then error(MAJOR_VERSION .. " requires AceHook-2.1") end
if not AceLibrary:HasInstance("BagLib-1.0") then error(MAJOR_VERSION .. " requires BagLib-1.0") end

local lib = {}
AceLibrary("AceEvent-2.0"):embed(lib)
AceLibrary("AceHook-2.1"):embed(lib)
local baglib = AceLibrary("BagLib-1.0") 

function lib:PurgeMail(index, unusedflag)
  self.postalindex = index
  self.hasmoney = nil
  self.hasitem = nil
  self.autodelete = nil
  --self.disableinbox = nil
  local _, _, _, msgSubject, msgMoney, msgCOD, _, msgItem, _, _, msgText = GetInboxHeaderInfo(index)
  if msgCOD and msgCOD > 0 then self:TriggerEvent("Postal10_MailSkipped", index) return end
  if string.find(msgSubject, string.sub(AUCTION_SOLD_MAIL_SUBJECT,1,10)) 
    or string.find(msgSubject, string.sub(AUCTION_OUTBID_MAIL_SUBJECT,1,10)) 
    or string.find(msgSubject, string.sub(AUCTION_REMOVED_MAIL_SUBJECT,1,10))
    or string.find(msgSubject, string.sub(AUCTION_EXPIRED_MAIL_SUBJECT,1,10))
    or string.find(msgSubject, string.sub(AUCTION_WON_MAIL_SUBJECT ,1,10))
  then
    self.autodelete = 1
  end
  if not msgText and msgItem then
    self.autodelete = 1
  end    
  self.hasmoney = msgMoney
  self.hasitem = msgItem
  if self.hasitem then 
    self:TakeItem()
    return
  elseif self.hasmoney > 0 then
    self:TakeMoney()
    return
  else
    self:DeleteMail()
  end
end

function lib:TakeItem(index)
  if not index then 
    if self.postalindex then index = self.postalindex end
  end
  if not baglib:GetFreeSlot() then 
    self:TriggerEvent("Postal10_MailSkipped", index)
    return
  end
  self.skipitem = nil
  --self:DisableInbox(1)
  self:RegisterEvent("UNIT_INVENTORY_CHANGED")
  self:RegisterEvent("UI_ERROR_MESSAGE")
  TakeInboxItem(index)
end

function lib:GotItem()
  self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
  self:UnregisterEvent("UI_ERROR_MESSAGE")
  self.hasitem = nil
  if self.hasmoney and self.hasmoney > 0 then
    self:TakeMoney(self.postalindex)
  elseif self.skipitem then
    self:TriggerEvent("Postal10_MailSkipped", self.postalindex)
    self.skipitem = nil
    --self:DisableInbox(nil)
  elseif self.autodelete then
    self:TriggerEvent("Postal10_MailPurged", self.postalindex)	   
    --self:DisableInbox(nil)
  else
    self:DeleteMail(self.postalindex)
  end
end

function lib:TakeMoney(index)
  if not index then 
    if self.postalindex then
      index = self.postalindex
    end
  end
  --if not self.inboxdisabled then self:DisableInbox(1) end
  self:RegisterEvent("PLAYER_MONEY", once)
  TakeInboxMoney(index)
end

function lib:GotMoney()
  if not self.hasitem and not self.autodelete then
    self:DeleteMail()
  elseif self.hasitem then
    self:TriggerEvent("Postal10_MailSkipped", self.postalindex)
    --self:DisableInbox(nil)
  else
    self:TriggerEvent("Postal10_MailPurged", self.postalindex)	   
    --self:DisableInbox(nil)
  end
end 

function lib:DeleteMail(index)
  if not index then 
    if self.postalindex then
      index = self.postalindex
    end
  end
  --if not self.inboxdisabled then self:DisableInbox(1) end
  self:RegisterEvent("MAIL_INBOX_UPDATE")
  DeleteInboxItem(index)
end

function lib:MAIL_INBOX_UPDATE()
  --self:DisableInbox(nil)
  self:UnregisterEvent("MAIL_INBOX_UPDATE")
  self:ScheduleEvent("Postal10_MailPurged", 1, self.postalindex)
end 

function lib:ReturnMail(index)
  --self:DisableInbox(1)
  local _, _, _, _, _, _, _, _, _, wasReturned, _, canReply = GetInboxHeaderInfo(index)
  if not wasReturned and canReply then 
    self:RegisterEvent("MAIL_INBOX_UPDATE")
    ReturnInboxItem(index)
  else
    self:TriggerEvent("Postal10_MailSkipped", index)
  end
end

function lib:PLAYER_MONEY()
  self:GotMoney()
end

function lib:UNIT_INVENTORY_CHANGED() 
  self:GotItem()
end

function lib:UI_ERROR_MESSAGE(event)
  if arg1 == ERR_INV_FULL or arg1 == ERR_ITEM_MAX_COUNT or arg1 == ERR_MAIL_DATABASE_ERROR then 
    self.skipitem = 1
    self.GotItem()
  end 
end

-- based on Gmail 1.x which was ported from CT_Mailmod pre-1.5
function lib:DisableInbox(disable)
  if ( disable ) then
    for i = 1, 7 do
      getglobal("MailItem" .. i .. "ButtonIcon"):SetDesaturated(1)
    end
    if not self:IsHooked("InboxFrame_OnClick") then
      self:Hook("InboxFrame_OnClick", function() end, true)
    end
  else
    for i = 1, 7 do
      getglobal("MailItem" .. i .. "ButtonIcon"):SetDesaturated(nil)
    end
    if self:IsHooked("InboxFrame_OnClick") then
      self:Unhook("InboxFrame_OnClick")
    end
  end
end

-- based on Gmail 1.x which was ported from CT_Mailmod pre-1.5
function lib:DisableAttachments(disable)
  if disable then
    OpenMailMoneyButtonIconTexture:SetDesaturated(1)
    OpenMailPackageButtonIconTexture:SetDesaturated(1)
    if not self:IsHooked(OpenMailMoneyButton, "OnClick") then
      self:HookScript(OpenMailMoneyButton, "OnClick", function() end)
    end
    if not self:IsHooked(OpenMailPackageButton, "OnClick") then
      self:HookScript(OpenMailPackageButton, "OnClick", function() end)
    end
  else
    OpenMailMoneyButtonIconTexture:SetDesaturated(nil)
    OpenMailPackageButtonIconTexture:SetDesaturated(nil)
    if self:IsHooked(OpenMailMoneyButton, "OnClick") then
      self:Unhook(OpenMailMoneyButton, "OnClick")
    end
    if self:IsHooked(OpenMailPackageButton, "OnClick") then
      self:Unhook(OpenMailPackageButton, "OnClick") 
    end
  end
end

AceLibrary:Register(lib, MAJOR_VERSION, MINOR_VERSION)
lib = nil
