--[[ Postal_Forward: Add a Forward Button to Mails ]]--
Postal_Forward = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1")
local postal = AceLibrary("PostalLib-1.0")
local baglib = AceLibrary("BagLib-1.0")
local L = AceLibrary("AceLocale-2.2"):new("Postal_Forward")

L:RegisterTranslations("enUS", function() return {
["Forward"] = true,
["FW:"] = true
} end)

function Postal_Forward:OnEnable()
  self.Button = CreateFrame("Button", "OpenMailForwardButton", OpenMailFrame, "UIPanelButtonTemplate")
  self.Button:SetWidth(82)
  self.Button:SetHeight(22)
  self.Button:SetPoint("RIGHT", OpenMailReplyButton, "LEFT", 0, 0)
  self.Button:SetText(L["Forward"])
  self.Button.owner = self
  self.Button:SetScript("OnClick", function() this.owner:OpenForward() end)
  self:RegisterEvent("MAIL_CLOSED", "Reset")
end

function Postal_Forward:OpenForward()
  OpenMail_Reply()
  self.openmail = InboxFrame.openMailID
  local _, _, _, _, money = GetInboxHeaderInfo(self.openmail)
  if money and money > 0 then
    MoneyInputFrame_SetCopper(SendMailMoney, money)
    SetSendMailMoney(money)
  end
  SendMailNameEditBox:SetText("")
  local prefix = L["FW:"]
  local subject = OpenMailSubject:GetText()
  if ( strsub(subject, 1, strlen(prefix)) ~= prefix ) then
    subject = prefix.." "..subject
  end
  SendMailSubjectEditBox:SetText(subject or "")
  SendMailBodyEditBox:SetText(string.gsub(OpenMailBodyText:GetText() or "", "\n", "\n>"))
  SendMailNameEditBox:SetFocus()
  HideUIPanel(OpenMailFrame)
  self:ToggleForward(1)
end

function Postal_Forward:SendMailMailButton_OnClick()
  self.invtable = nil
  if GetInboxItem(self.openmail) then 
    self.invtable = baglib:GetSlotTable()
  end
  self:RegisterEvent("Postal10_MailSkipped", "Reset", once)
  self:RegisterEvent("Postal10_MailPurged", "Send", once)
  postal:PurgeMail(self.openmail)
  SendMailMailButton:Disable()
end

function Postal_Forward:Send()
  if self.invtable then
    local bag, slot = baglib:FindTakenSlot(self.invtable)
    PickupContainerItem(bag, slot)
    ClickSendMailItemButton()
  end
  self:RegisterEvent("MAIL_SEND_SUCCESS", "Reset")
  SendMailFrame_SendMail()
  SendMailMailButton:Disable()
end

function Postal_Forward:InboxFrame_OnClick(index)
  self.hooks["InboxFrame_OnClick"](index)
  local _, _, _, _, _, COD = GetInboxHeaderInfo(index)
  if COD and COD > 0 then
    self.Button:Disable()
  else
    self.Button:Enable()
  end
end

function Postal_Forward:ToggleForward(toggle)
  if toggle then
    self.Button:Disable()
    if not self:IsHooked(SendMailPackageButton, "OnEnter") then self:HookScript(SendMailPackageButton, "OnEnter", function() end) end
    SendMailCODButton:Disable()
    postal:DisableAttachments(1)
    if not self:IsHooked("SendMailMailButton_OnClick") then self:Hook("SendMailMailButton_OnClick", true) end
    if not self:IsHooked("SendMailPackageButton_OnClick") then self:Hook("SendMailPackageButton_OnClick", function() end, true) end
  else
    self.Button:Enable()
    if self:IsHooked(SendMailPackageButton, "OnEnter") then self:Unhook(SendMailPackageButton, "OnEnter") end
    SendMailCODButton:Enable()
    postal:DisableAttachments(nil)
    if self:IsHooked("SendMailMailButton_OnClick") then self:Unhook("SendMailMailButton_OnClick") end
    if self:IsHooked("SendMailPackageButton_OnClick") then self:Unhook("SendMailPackageButton_OnClick") end
  end
end

function Postal_Forward:Reset()
  self:ToggleForward()
  if self:IsEventRegistered("MAIL_SEND_SUCCESS") then self:UnregisterEvent("MAIL_SEND_SUCCESS") end
  if self:IsEventRegistered("Postal10_MailSkipped") then self:UnregisterEvent("Postal10_MailSkipped") end
  if self:IsEventRegistered("Postal10_MailPurged") then self:UnregisterEvent("Postal10_MailPurged") end
  MailFrameTab_OnClick(1)
end 
