--[[ Postal_Select: Adds checkboxes to the inbox with buttons for returning/opening. ]]--
Postal_Select = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1")
local postal = AceLibrary("PostalLib-1.0")

function Postal_Select:OnEnable()
  --create the open button
  self.OpenButton = CreateFrame("Button", "PostalSelectOpenButton", InboxFrame, "UIPanelButtonTemplate")
  self.OpenButton:SetWidth(120)
  self.OpenButton:SetHeight(25)
  self.OpenButton:SetPoint("RIGHT", InboxFrame, "TOP", 5, -53)
  self.OpenButton:SetText("Open")
  self.OpenButton:SetScript("OnClick", function() Postal_Select:HandleSelect(1) end)

  --create the return button
  self.ReturnButton = CreateFrame("Button", "PostalSelectReturnButton", InboxFrame, "UIPanelButtonTemplate")
  self.ReturnButton:SetWidth(120)
  self.ReturnButton:SetHeight(25)
  self.ReturnButton:SetPoint("LEFT", InboxFrame, "TOP", 10, -53)
  self.ReturnButton:SetText("Return")
  self.ReturnButton:SetScript("OnClick", function() Postal_Select:HandleSelect() end)

  --indent to make room for the checkboxes
  MailItem1:SetPoint("TOPLEFT", "InboxFrame", "TOPLEFT", 48, -80)
  for i = 1, 7 do
    _G["MailItem"..i.."ExpireTime"]:SetPoint("TOPRIGHT", "MailItem"..i, "TOPRIGHT", 10, -4)
    _G["MailItem"..i]:SetWidth(280)
  end
  
  --now create the checkboxes
  for i = 1, 7 do
     local CB = CreateFrame("CheckButton", "PostalInboxCB"..i, _G["MailItem"..i], "OptionsCheckButtonTemplate")
     CB:SetID(i)
     CB:SetPoint("RIGHT", "MailItem"..i, "LEFT", 1, 0)
     CB:SetWidth(24)
     CB:SetHeight(24)
     CB:SetHitRectInsets(0, 0, 0, 0)
     CB:SetScript("OnClick", function() Postal_Select:ToggleMail(this) end)
  end

  --hook shizz
  self:Hook("InboxFrame_Update", true)
  --empty table
  self.SelectedMail = { }
  --make sure to reset on mail close
  self:RegisterEvent("MAIL_CLOSE", "Reset")
end

function Postal_Select:ToggleMail(frame)
  local index = frame:GetID() + (InboxFrame.pageNum-1)*7
  if not frame:GetChecked() then
    self:tremovebyval(self.SelectedMail, index)
  else
    tinsert(self.SelectedMail, index)
  end
end

function Postal_Select:HandleSelect(mode)
  if not self.SelectedMail[1] then return end
  self.mode = mode
  postal:DisableInbox(1)
  self.toprocess = table.getn(self.SelectedMail)
  table.sort(self.SelectedMail)
  self.indexadjust = 0
  self:RegisterEvent("Postal10_MailPurged")
  self:RegisterEvent("Postal10_MailSkipped", "ProcessNext")
  if self.mode then 
    postal:PurgeMail(self.SelectedMail[1])
    return
  else
    postal:ReturnMail(self.SelectedMail[1])
    return
  end
end

function Postal_Select:Postal10_MailPurged()
  self.indexadjust = self.indexadjust + 1
  Postal_Select:ProcessNext()
end

function Postal_Select:ProcessNext()
  InboxFrame_Update()
  self.toprocess = self.toprocess - 1
  if self.toprocess > 0 then 
    tremove(self.SelectedMail, 1)
    self.SelectedMail[1] = self.SelectedMail[1] - self.indexadjust
    if self.mode then 
      postal:PurgeMail(self.SelectedMail[1])
    else
      postal:ReturnMail(self.SelectedMail[1])
    end
  else
    self:Reset()
  end
end


function Postal_Select:InboxFrame_Update()
  self.hooks["InboxFrame_Update"]()
  for i = 1, 7 do
    local index = i + (InboxFrame.pageNum-1)*7
    local CB = _G["PostalInboxCB"..i]
    if index > GetInboxNumItems() then
      CB:Hide()
    else
      CB:Show()
      CB:SetChecked(nil)
      for key, value in pairs(self.SelectedMail) do
        if value == index then
          CB:SetChecked(1)
	  break
        end
      end
    end
  end
end

function Postal_Select:Reset()
  self:UnregisterEvent("Postal10_MailPurged")
  self:UnregisterEvent("Postal10_MailSkipped")
  self.SelectedMail = { }
  self.toprocess = 1
  postal:DisableInbox()
  InboxFrame_Update()
end

--based on a free function at wowwiki
function Postal_Select:tremovebyval(tab, val)
   for k,v in pairs(tab) do
     if v == val then
       table.remove(tab, k)
       return true
     end
   end
   return false
end
